/*      PC MONITOR PROGRAM FOR                                  */
/*      MOTOROLA 68HC11 MICROCONTROLLERS                        */
/*      BY:             PHILIP DEVRIES                          */
/*      VERSION:        3.0                                     */
/*      DATE:           DECEMBER 1, 1994                        */
/*      COMPILED BY:    QUICK-C 2.0 BY MICROSOFT                */

#include <ctype.h>
#include <stdio.h>
#include <io.h>
#include <bios.h>
#include <conio.h>
#include <string.h>
#include <stdlib.h>
#include <graph.h>


/*MEMORY EDIT AND DISPLAY PROCEEDURES*/

void dump_memory(void);                     /*COMMAND 'D'*/
void edit_memory(void);                     /*COMMAND 'E'*/
void load_file(void);                       /*COMMAND 'L'*/
void change_prom_mode(void);                /*COMMAND 'P'*/

/*REGISTER EDIT AND DISPLAY PROCEEDURES*/

void edit_registers(void);                  /*COMMAND 'R'*/

/*BREAKPOINT EDIT AND DISPLAY PROCEEDURES*/

void edit_breakpoints(void);                /*COMMAND 'B'*/

/*6811 PROGRAM RUN PROCEEDURES*/

void go(void);                              /*COMMAND 'G'*/
void step(void);                            /*COMMAND 'S'*/
void trace(void);                           /*COMMAND 'T'*/
  unsigned char step_multiple(unsigned char);
  void send_breakpoints(void);
  void receive_breakpoints(void);
  void send_stack(void);
  void receive_stack(void);
  unsigned char run_and_terminate(void);

/*STARTUP PROCEEDURES*/

int setup_6811_monitor(void);               /*COMMAND 'M'*/
	int load_6811_bootstrap(void);
	int load_6811_monitor(void);
	int load_6811_vectors(void);
	void choose_register_location(void);
	void find_register_location(void);
	void find_6811_mode(void);

/*SCREEN DISPLAY PROCEEDURES*/

void redraw_screen(void);                   /*COMMAND 'Z'*/
void help_toggle(void);                     /*COMMAND 'H'*/
	void display_memory(void);
	void display_program(void);
	void display_stack(void);
	void display_register_block(void);
	void display_registers(void);
	void display_breakpoints(void);
	void clear_status(void);
	void clear_mode_status(void);
	void clear_help(void);
	void clear_command(void);
	void display_command_summary(void);
	void print_hex_2byte(unsigned);
	void print_hex_byte(unsigned char);

/*KEYBOARD INPUT PROCEEDURES*/

	unsigned input_two_bytes( unsigned *two_byte);
	unsigned input_one_byte(unsigned char *one_byte);
	int input_hex_character(void);
	unsigned input_two_digits(unsigned char *two_digits);
	int input_decimal_character(void);

/*RS232 PROCEEDURES  */

	void send_two_bytes(unsigned);
	void send(unsigned char);
	unsigned char receive(void);
	unsigned receive_status(void);
	void clear_rs232(void);

#define nohit                   0

#define on                      0
#define off                     1


#define success                 0
#define fail                    1

#define no_mode                 0
#define single_chip_mode        1
#define bootstrap_mode          2
#define expanded_mode           3
#define test_mode               4

#define eeprom_edit             4
#define eprom_edit              5
#define ram_edit                6

#define TMSK1                   0x0022
#define TMSK2                   0x0024
#define INIT                    0x003d
#define HPRIO                   0x003c
#define CONFIG                  0x003f

#define ascii_validhex          0xff
#define ascii_decimal           0xee
#define ascii_backspace         0x08
#define ascii_return            0x0D
#define ascii_arrow             0x00
#define ascii_space             0x20
#define ascii_left_arrow        0x4b
#define ascii_up_arrow          0x48
#define ascii_right_arrow       0x4d
#define ascii_down_arrow        0x50


unsigned int    display_address         = 0x0000,
				current_address         = 0x0000,
				register_block_address  = 0x1000;
unsigned int    SP                      = 0x00FF,
				IX                      = 0x0000,
				IY                      = 0x0000,
				PC                      = 0x0040;
unsigned char   CCR                     = 0xD0,
				ACCB                    = 0x00,
				ACCA                    = 0x00;
unsigned int    breakpoint[10]  = { 0xffff,0xffff,0xffff,0xffff,0xffff,
									0xffff,0xffff,0xffff,0xffff,0xffff };
unsigned char   breakpoint_data[10];

unsigned int    _6811mode   = no_mode,
				prom_mode   = ram_edit,
				help_mode   = on,
				auto_update = on;

/* ********************************************** */
/* Main routine starts by initializing the        */
/* screen.  Then waits for input and jumps        */
/* to appropriate proceedure                      */
/* ********************************************** */

void main(void)
{
	int character,flag,status_and_data;

	/*CHECK IF MONITOR PROGRAM IS ALREADY RUNNING*/
	/*SELECT MODE IF NOT*/

	flag = setup_6811_monitor();
	if (flag == fail)
		return;

	_bios_serialcom(_COM_INIT,0,_COM_CHR8 | _COM_STOP1 |
					_COM_NOPARITY |_COM_9600);

	redraw_screen();

	while (1)
	{
		clear_command();
		while (kbhit() == nohit)
		   clear_rs232();
		character = getch();
		switch( character )
		{
			case 0x00:
				getch();
				break;
			case 'a': case 'A':
				if (_6811mode == bootstrap_mode)
					break;
				clear_status();
				switch (auto_update)
				{
					case on:
						auto_update = off;
						printf("Automatic memory display off");
						break;
					case off:
						auto_update = on;
						printf("Automatic memory display on");
						break;
				}
				break;

			case 'd': case 'D':
				printf("D");
				dump_memory();
				break;
			case 'e': case 'E':
				printf("E");
				edit_memory();
				break;
			case 'r': case 'R':
				if (_6811mode == bootstrap_mode   )
					break;
				printf("R");
				edit_registers();
				break;
			case 'b' : case 'B':
				if (_6811mode == bootstrap_mode   )
					break;
				printf("B");
				edit_breakpoints();
				break;
			case 'g': case 'G':
				if (_6811mode == bootstrap_mode   )
					break;
				printf("G");
				go();
				break;
			case 'h': case 'H':
				help_toggle();
				break;
			case 'm': case 'M':
				flag = setup_6811_monitor();
				if (flag == fail)
				   return;
				redraw_screen();
				break;
			case 'p': case 'P':
				if(_6811mode != bootstrap_mode)
					break;
				change_prom_mode();
				break;
			case 's': case 'S':
				if (_6811mode == bootstrap_mode   )
					break;
				step();
				break;
			case 'l': case 'L':
				printf("L");
				load_file();
				break;
			case 't': case 'T':
				if (_6811mode == bootstrap_mode)
					break;
				printf("T");
				trace();
				break;
			case 'z': case 'Z':
				redraw_screen();
				break;
			case 'q': case 'Q':
				return;
		}
	}
}

/* ********************************************** */
/* Memory display routine prompts user for a         */
/* starting address.  128 bytes are then          */
/* displayed starting at that address             */
/* ********************************************** */

void dump_memory( void )
{
	unsigned int key_status = ascii_space;

	display_address = display_address + 0x0080;
	clear_status();
	printf("Display memory");
	clear_help();
	printf("Enter starting address");
	_settextposition(25,13);
	while (key_status == ascii_space)
		 key_status = input_two_bytes(&display_address);
	clear_help();
	clear_status();
	switch(key_status)
	{
		case ascii_backspace:
			return;
		default:
			display_memory();
	}
}

/* ********************************************** */
/* Edit memory routine prompts user for a         */
/* starting address.  The user can then edit      */
/* memory locations                               */
/* If eeprom programming is enabled, this routine */
/* will also edit eeprom memory locations.        */
/* ********************************************** */

void edit_memory( void )
{
   short row, col;
   unsigned int key_status = ascii_space, test_disp_addr, test_curr_addr;
   unsigned char data_byte;

   clear_status();
   printf("Editing memory");
   clear_help();
   printf("Enter address");
   _settextposition(25,13);

   while (key_status == ascii_space)
		key_status = input_two_bytes(&display_address);

   clear_help();
   clear_command();

   if (key_status == ascii_backspace)
		return;

   current_address = display_address;
   display_memory();

   while(1)
   {
	  if ((display_address < 0x0080) | (display_address >= 0xff80))
	  {
		test_disp_addr = display_address + 0x8000;
		test_curr_addr = current_address + 0x8000;
	  }
	  else
	  {
		test_disp_addr = display_address;
		test_curr_addr = current_address;
	  }

	  if (test_curr_addr < test_disp_addr)
	  {
		 display_address = current_address - 0x0070;
		 display_memory();
	  }

	  if (test_curr_addr >= (test_disp_addr + 0x0080))
	  {
		display_address = current_address;
		display_memory();
	  }

	  send('A');
	  send_two_bytes(current_address);

	  send('D');
	  data_byte = receive();

	  row = 1+((current_address & 0xfff0)-display_address)/16;
	  col = current_address - (current_address & 0xfff0) + 1;
	  _settextposition( row + 2, col *3 + 4);

	  key_status = input_one_byte(&data_byte);

	  switch (key_status)
	  {
		 case ascii_validhex:
			switch(prom_mode)
			{
			   case eeprom_edit:
				  send('P');
				  send(data_byte);
				  data_byte = receive();
				  break;
			   default:
				  send('E');
				  send(data_byte);
				  send('D');
				  data_byte = receive();
			}
			_settextposition( row + 2, col * 3 + 4);
			print_hex_byte(data_byte);
			_settextposition( row + 2, col +  55);
			if((data_byte > 0x1f) && (data_byte < 0x7f))
				printf("%c",data_byte);
			else
				printf(".");
		 case ascii_space: case ascii_right_arrow:
			current_address++;
			break;
		 case ascii_left_arrow:
			current_address--;
			break;
		 case ascii_up_arrow:
			current_address = current_address - 0x10;
			break;
		 case ascii_down_arrow:
			current_address = current_address + 0x10;
			break;
		 case ascii_return : case ascii_backspace :
			clear_status();
			return;

	  }
   }
}

/* ********************************************** */
/*      Load data from file into 6811 memory      */
/*      file must be motorola S record file       */
/* ********************************************** */

void load_file(void)
{
   char byte[] = "00";
   char two_byte[] = "0000";
   char data;
   char *byte_ptr = &byte[0];
   char *two_byte_ptr = &two_byte[0];
   char file_name[40];
   int character,i,num_data;
   FILE *fp;
   unsigned int current1_address;

   /*GET THE FILENAME*/
   current1_address = current_address;
   clear_status();
   printf("Load file from disk");
   clear_help();
   printf("Enter filename. S-record file");
   _settextposition(23,1);
   printf("FILENAME>                                ");
   _settextposition(23,11);
   scanf("%40s",file_name);
   clear_help();
   clear_command();

   /* DO SUBROUTINE IF FILE CAN BE OPENED */
   /* ELSE FLAG AS AN ERROR               */

   if(fp = fopen( file_name , "rb") )
   {
	  clear_status();
	  printf("Loading file");
	  while(1)
	  {
		 character = fgetc( fp );

		 /*THE FIRST CHARACTER ON EVERY LINE MUST BE AN S   */
		 /*ELSE FLAG AS AN ERROR                            */

		 if (character == 'S')
		 {
			character = fgetc( fp );
			switch (character)
			{

			/*THE SECOND CHARACTER ON A DATA LINE IS 1  */
			/*MEANS THE LINE CONTAINS DATA              */

			case '1' :

			   /*THE THIRD AND FOURTH CHARACTERS ON A DATA LINE   */
			   /*INDICATE THE NUMBER OF BYTES, INCLUDING CHECKSUM */

			   character = fgetc ( fp );       /* THIRD BYTE       */
			   if(isxdigit(character) == 0)    /* MUST BE HEX      */
				  goto S_record_error;
			   byte_ptr = &byte[0];
			   byte[0] = (char)character;
			   character = fgetc ( fp );       /* FOURTH BYTE      */
			   if(isxdigit(character) == 0)    /* MUST BE HEX      */
				  goto S_record_error;
			   byte[1] = (char)character;
			   num_data = (int)strtol(byte_ptr, &byte_ptr,16) - 3;

			   /*THE FIFTH THROUGH EIGTH CHARACTERS ON A DATA LINE */
			   /*INDICATE THE STARTING ADDRESS                     */

			   i = 0;
			   while(i < 4)
			   {
				  character = fgetc ( fp );      /* GET FOUR BYTES   */
				  if(isxdigit(character) == 0)   /* EACH MUST BE HEX */
					 goto S_record_error;
				  two_byte[i] = (char)character;
				  i++;
			   }
			   two_byte_ptr = &two_byte[0];
			   current_address = (unsigned int)strtol( two_byte_ptr,
						  &two_byte_ptr, 16);

			   /*NOW WE CAN EDIT THE NUMBER OF DATA STARTING AT ADDRESS */

			   send('A');
			   send_two_bytes(current_address);
			   while(num_data > 0)
			   {
				  character = fgetc ( fp );
				  if(isxdigit(character) == 0)    /* EACH BYTE MUST BE */
					 goto S_record_error;         /* A HEX DIGIT       */
				  byte[0] = (char)character;
				  character = fgetc ( fp );
				  if(isxdigit(character) == 0)
					 goto S_record_error;
				  byte[1] = (char)character;      /* CONVERT TWO DATA */
				  byte_ptr = &byte[0];            /* FROM FILE TO HEX */
				  data = (char)strtol(byte_ptr, &byte_ptr,16);

				  switch(prom_mode)
				  {
					case ram_edit:
						send('C');
						send(data);
						break;
					case eeprom_edit:
						send('P');
						send(data);
						receive();
						break;
					case eprom_edit:
						send('O');
						send(data);
						receive();
						break;
				  }
				  num_data--;
				  current_address++;
			   }
			   /*CHECKSUM AND END OF LINE,LINE FEED AT END OF LINE */
			   /*JUST DISCARD THIS DATA */

			   character = fgetc ( fp );
			   character = fgetc ( fp );
			   character = fgetc ( fp );
			   character = fgetc ( fp );
			   break;

			/*SECOND CHARACTER ON LINE A 9 MEANS END OF FILE*/

			case '9' :
			   clear_status();
			   printf("File loaded correctly");
			   display_memory();
			   fclose(fp);
			   current_address = current1_address;
			   return;

			/*SECOND CHARACTER NOT 9 OR 1 MEANS FILE IS BAD */

			default :
			   goto S_record_error;

			} /* END OF SWITCH */

		 }  /* END OF IF CONDITION */

		 /* CASE 1ST CHARACTER ON LINE NOT S */

		 else
			goto S_record_error;

	  }  /* END OF MAIN WHILE LOOP                    */

	  /* CASE ANY ERROR FLAGGED WHILE READING FILE */

	  S_record_error:
	  clear_status();
	  printf("Bad S-Record file");
	  fclose(fp);
	  current_address = current1_address;
	  return;
   }

   /* CASE FILE NOT READABLE */
   /* END OF IF CONDITION    */

   else
   clear_status();
   printf("Unable to open file");
}


void change_prom_mode(void)
{
	clear_mode_status();
	switch (prom_mode)
	{
		case ram_edit:
			prom_mode = eprom_edit;
			printf("Bootstrap - Eprom editor ");
			break;
		case eprom_edit:
			prom_mode = eeprom_edit;
			printf("Bootstrap - EEprom editor");
			break;
		case eeprom_edit:
			prom_mode = ram_edit;
			printf("Bootstrap - Ram editor   ");
			break;
	}
}



/* ********************************************** */
/*      register editor                           */
/* ********************************************** */

void edit_registers( void )
{
	unsigned int key_status;
	int character;

	clear_status();
	printf("Editing registers");
	clear_help();
	printf("Enter name of register");

	_settextposition(25,13);
	character = getch();

	clear_help();
	printf("Space = skip  ENTER = end edit");

	switch(character)
	{
		case 'a': case 'A':
	start_reg:
			_settextposition(13,10);
			key_status = input_one_byte( &ACCA);
			if ((key_status == ascii_return) |
				(key_status == ascii_backspace))
			  break;
		case 'b': case 'B':
			_settextposition(13,25);
			key_status = input_one_byte( &ACCB);
			if ((key_status == ascii_return) |
				(key_status == ascii_backspace))
			  break;
		case 'x': case 'X':
			_settextposition(14,10);
			key_status = input_two_bytes( &IX);
			if ((key_status == ascii_return) |
				(key_status == ascii_backspace))
			  break;
		case 'y': case 'Y':
			_settextposition(14,25);
			key_status = input_two_bytes( &IY);
			if ((key_status == ascii_return) |
				(key_status == ascii_backspace))
			  break;
		case 'p': case 'P':
			_settextposition(13,40);
			key_status = input_two_bytes( &PC);
			if ((key_status == ascii_return) |
				(key_status == ascii_backspace))
			  break;
			display_program();

		case 's': case 'S':
			_settextposition(14,40);
			key_status = input_two_bytes( &SP);
			if ((key_status == ascii_return) |
				(key_status == ascii_backspace))
			  break;
			display_stack();
			goto start_reg;
	}
	clear_help();
}

/* ********************************************** */
/*      Breakpoint editor                         */
/* ********************************************** */

void edit_breakpoints(void)
{
	int i, character;
	unsigned int key_status;

	clear_help();
	if (help_mode == on)
	{
		printf("toggle help mode off");
		return;
	}

	printf("'c' to clear breakpoints");

	_settextposition(25,13);
	character = getch();

	clear_help();
	switch(character)
	{
	case 'c': case 'C':
		i = 9;
		while(i >= 0)
		{
		  breakpoint[i] = 0xffff;
		  i--;
		}
		display_breakpoints();
		return;
	case ascii_backspace:
		return;
	default:
		printf("Space = skip  ENTER = end edit");
		clear_command();
		i = 9;
		while(i >= 5)
		{
		   _settextposition(23,(9-i)*5 + 51);
		   key_status = input_two_bytes(&breakpoint[i]);
		   switch (key_status)
		   {
			  case ascii_return : case ascii_backspace:
				 clear_help();
				 return;
		   }
		   i--;
		}
		while(i >= 0)
		{
		   _settextposition(24,(4-i)*5 + 51);
		   key_status = input_two_bytes(&breakpoint[i]);
		   switch (key_status)
		   {
			  case ascii_return : case ascii_backspace:
				 clear_help();
				 return;
		   }
		   i--;
		}
	}
	clear_help;
}

/* ********************************************** */
/*      begin execution of user program           */
/* ********************************************** */

void go(void)
{
   unsigned char break_character, tmsk1_data;

   if (getch() == ascii_backspace)
	  return;
   clear_command();
   clear_status();
   printf("Program running... X to stop.");
   send_breakpoints();

   send('A');
   send_two_bytes(register_block_address + TMSK1);      /*disable the OC5*/
   send('D');                                           /*timer interrupt*/
   tmsk1_data = receive();                              /*mask           */
   tmsk1_data = tmsk1_data & 0xf7;
   send('E');
   send(tmsk1_data);

   send_stack();
   break_character = run_and_terminate();

   if (break_character == 'S')
		printf("Timer OC5 interrupt");

   if (break_character != 'R')
	   receive_stack();

   if (break_character == 'B')
	   PC--;
   receive_breakpoints();
   display_registers();
   display_program();
   if (auto_update == on)
	  display_memory();
   else
   {
	  clear_help();
	  printf("Displayed memory may have changed");
   }
}

/* ********************************************** */
/*      single step user program                  */
/* ********************************************** */

void step(void)
{
	unsigned char break_character;
	clear_status();
	break_character = step_multiple(1);
	if (break_character == 'S');
		printf("One instruction executed");
	display_registers();
	display_program();
	if (auto_update == on)
	   display_memory();
	else
	{
		clear_help();
		printf("Displayed memory may have changed");
	}
}

void trace(void)
{
	unsigned char break_character, numsteps = 0;
	unsigned int key_status;
	clear_help();
	printf("Enter number of steps 0 - 99");
	_settextposition(25,13);
	key_status = input_two_digits(&numsteps);
	clear_help();

	if (key_status != ascii_decimal)
		return;
	if (numsteps == 0)
		return;

	printf("Program running -- %d steps",numsteps);

	clear_status();
	break_character = step_multiple(numsteps);
	clear_help();
	if (break_character == 'S')
	{
		clear_status();
		printf("%d instructions executed",numsteps);
	}
	display_registers();
	display_program();
	if (auto_update == on)
	   display_memory();
	else
	{
		clear_help();
		printf("Displayed memory may have changed");
	}
}

unsigned char step_multiple(unsigned char numsteps)
{
   unsigned char break_character, tmsk1_data, opcode, CCRb, i=0;
   unsigned char tmsk2_data, tmsk2_data_back;
   send('A');                                           /*enable the   */
   send_two_bytes(register_block_address + TMSK1);      /*interrupt OC5*/
   send('D');                                           /*mask         */
   tmsk1_data = receive();
   tmsk1_data = tmsk1_data | 0x08;
   send('E');

   send(tmsk1_data);                                    /*if in test*/
   send('A');                                           /*mode, set timer*/
   send_two_bytes(register_block_address + TMSK2);      /*rate to 500nS*/
   send('D');
   tmsk2_data = receive();
   tmsk2_data_back = tmsk2_data;
   tmsk2_data = tmsk2_data & 0xfc;
   send('E');
   send(tmsk2_data);

   while (numsteps > i)
   {
	send('A');                   /*SEI and CLI instructions are not */
	send_two_bytes(PC);          /*executed, but simulated.  This is*/
	send('D');                   /*because single step function uses*/
	opcode = receive();          /*the interrupt enable flag for    */
	switch(opcode)               /*itself                           */
	{
		case 0x0F:
			CCR = CCR | 0x10;
			PC++;
			i++;
			break;
		case 0x0E:
			CCR = CCR & 0xEF;
			PC++;
			i++;
	}



	CCRb = CCR;                              /*Enable the interrupts*/
	CCR = CCR & 0xEF;
	send_stack();
	break_character = run_and_terminate();
	receive_stack();
											 /*unless the instruction was  */
	if(opcode != 0x3b)                       /*RTI ( which loads CCR from  */
		CCR = (CCRb & 0x10) | (CCR & 0xEF);  /*stack, restore old interrupt*/
	i++;                                     /*enable bit*/
	if (break_character != 'S')
	   i = numsteps;
   }

   /*restore timer rates in test mode*/

   send('A');
   send_two_bytes(register_block_address + TMSK2);
   send('D');
   tmsk2_data = receive();
   tmsk2_data = (tmsk2_data & 0xfc) | (tmsk2_data_back & 0x03);
   send('E');
   send(tmsk2_data);

   return break_character;
}


void send_breakpoints(void)
{
   int i = 0;
   while(i <= 9)
   {
	  if (PC == breakpoint[i])
		 step_multiple(1);
	  send('A');
	  send_two_bytes(breakpoint[i]);
	  send('D');
	  breakpoint_data[i] = receive();
	  send('E');
	  send(0x3f);
	  i++;
   }
}

void receive_breakpoints(void)
{
   int i = 0;
   while(i <= 9)
   {
	  current_address = breakpoint[i];
	  send('A');
	  send_two_bytes(current_address);
	  send('E');
	  send(breakpoint_data[i]);
	  i++;
	}
}

void send_stack(void)
{
   SP = SP - 8;

   send('A');
   send_two_bytes(SP);

   send('C');
   send(CCR);

   send('C');
   send(ACCB);

   send('C');
   send(ACCA);

   send('C');
   send((char)((IX >> 8) & 0x00ff));       /*  IXH  */

   send('C');
   send((char)(IX & 0x00ff));              /*  IXL  */

   send('C');
   send((char)((IY >> 8) & 0x00ff));       /*  IYH  */

   send('C');
   send((char)(IY & 0x00ff));              /*  IYL  */

   send('C');
   send((char)((PC >> 8) & 0x00ff));       /*  PCH  */

   send('C');
   send((char)(PC & 0x00ff));              /*  PCL  */

   send('A');
   send_two_bytes(SP);
}

void receive_stack(void)
{
   char low, high;
   send('M');
   CCR  = receive();
   send('M');
   ACCB = receive();
   send('M');
   ACCA = receive();
   send('M');
   high  = receive();      /*  IXH  */
   send('M');
   low  = receive();       /*  IXL  */
   IX = (((unsigned)high << 8) | ((unsigned)low & 0x00ff));
   send('M');
   high  = receive();      /*  IYH  */
   send('M');
   low  = receive();       /*  IYL  */
   IY = (((unsigned)high << 8) | ((unsigned)low & 0x00ff));
   send('M');
   high  = receive();      /*  PCH  */
   send('M');
   low  = receive();       /*  PCL  */
   PC = (((unsigned)high << 8) | ((unsigned)low & 0x00ff));
   send('S');
   low  = receive();      /*  SPL  */
   high= receive();       /*  SPH  */
   SP = (((unsigned)high << 8) | ((unsigned)low & 0x00ff));
   SP = SP - 1;
}


unsigned char run_and_terminate(void)
{  int character;
   unsigned char break_character = 0x00;

   send('G');

   while (break_character == 0x00)
   {
	  if (kbhit() != nohit)
	  {
		 character = getch();
		 if((character == 'x') | (character == 'X'))
				send('X');
	  }
	  if ((receive_status() & 0x9f00) == 0x0100)
	  {
		 break_character = receive();
		 switch(break_character)
		 {
		 case 'X':
			clear_status();
			printf("Program halted by user");
			break;
		 case 'B' :
			clear_status();
			printf("Program halted by breakpoint");
			break;
		 case 'S' :
			break;
		 case 'O' :
			clear_status();
			printf("Illegal opcode detected");
			break;
		 case 'T' :
			clear_status();
			printf("System timer interrupt");
			break;
		 case 'I' :
			clear_status();
			printf("Maskable interrupt occured");
			break;
		 case 'M' :
			clear_status();
			printf("Nonmaskable interrupt occured");
			break;
		 case 'R':
			clear_status();
			printf("SYSTEM RESET");
			SP = SP + 9;
			break;
		 default:
			clear_status();
			printf("ERROR Unknown termination");
			_settextposition(20,37);
			print_hex_byte(break_character);
			break;
		 }
	  }
   }
   return break_character;
}

void choose_register_location()
{
	int character;
	unsigned char init_data;
	_clearscreen( _GCLEARSCREEN );
	printf("Register location       Internal ram location\n\n");
	printf("0:      0000 - 003F     0:     0000 - 00FF\n");
	printf("1:      1000 - 103F     1:     1000 - 10FF\n");
	printf("2:      2000 - 203F     2:     2000 - 20FF\n");
	printf("3:      3000 - 303F     3:     3000 - 30FF\n");
	printf("4:      4000 - 403F     4:     4000 - 40FF\n");
	printf("5:      5000 - 503F     5:     5000 - 50FF\n");
	printf("6:      6000 - 603F     6:     6000 - 60FF\n");
	printf("7:      7000 - 703F     7:     7000 - 70FF\n");
	printf("8:      8000 - 803F     8:     8000 - 80FF\n");
	printf("9:      9000 - 903F     9:     9000 - 90FF\n");
	printf("A:      A000 - A03F     a:     A000 - A0FF\n");
	printf("B:      B000 - B03F     b:     B000 - B0FF\n");
	printf("C:      C000 - C03F     c:     C000 - C0FF\n");
	printf("D:      D000 - D03F     d:     D000 - D0FF\n");
	printf("E:      E000 - E03F     e:     E000 - E0FF\n");
	printf("F:      F000 - F03F     f:     F000 - F0FF\n");
	printf("Choose 68HC11 register location\n\n");
	character = getch();
	switch (character)
	{
		case '0':
			register_block_address = 0x0000;
			init_data = 0x00;
			break;
		case '1':
			register_block_address = 0x1000;
			init_data = 0x01;
			break;
		case '2':
			register_block_address = 0x2000;
			init_data = 0x02;
			break;
		case '3':
			register_block_address = 0x3000;
			init_data = 0x03;
			break;
		case '4':
			register_block_address = 0x4000;
			init_data = 0x04;
			break;
		case '5':
			register_block_address = 0x5000;
			init_data = 0x05;
			break;
		case '6':
			register_block_address = 0x6000;
			init_data = 0x06;
			break;
		case '7':
			register_block_address = 0x7000;
			init_data = 0x07;
			break;
		case '8':
			register_block_address = 0x8000;
			init_data = 0x08;
			break;
		case '9':
			register_block_address = 0x9000;
			init_data = 0x09;
			break;
		case 'A': case 'a':
			register_block_address = 0xA000;
			init_data = 0x00;
			break;
		case 'B': case 'b':
			register_block_address = 0xB000;
			init_data = 0x0B;
			break;
		case 'C': case 'c':
			register_block_address = 0xC000;
			init_data = 0x0C;
			break;
		case 'D': case 'd':
			register_block_address = 0xD000;
			init_data = 0x0D;
			break;
		case 'E': case 'e':
			register_block_address = 0xE000;
			init_data = 0x0E;
			break;
		case 'F': case 'f':
			register_block_address = 0xF000;
			init_data = 0x0F;
			break;
		default:
			register_block_address = 0x0000;
			init_data = 0x00;
			break;
	}
	printf("Choose 68HC11 internal ram location\n");
	character = getch();
	switch (character)
	{
		case '0':
			init_data = init_data | 0x00;
			break;
		case '1':
			init_data = init_data | 0x10;
			break;
		case '2':
			init_data = init_data | 0x20;
			break;
		case '3':
			init_data = init_data | 0x30;
			break;
		case '4':
			init_data = init_data | 0x40;
			break;
		case '5':
			init_data = init_data | 0x50;
			break;
		case '6':
			init_data = init_data | 0x60;
			break;
		case '7':
			init_data = init_data | 0x70;
			break;
		case '8':
			init_data = init_data | 0x80;
			break;
		case '9':
			init_data = init_data | 0x90;
			break;
		case 'A':
			init_data = init_data | 0xA0;
			break;
		case 'B':
			init_data = init_data | 0xB0;
			break;
		case 'C':
			init_data = init_data | 0xC0;
			break;
		case 'D':
			init_data = init_data | 0xD0;
			break;
		case 'E':
			init_data = init_data | 0xE0;
			break;
		case 'F':
			init_data = init_data | 0xF0;
			break;
		default:
			init_data = init_data | 0x00;
			break;
	}
	send('A');
	send_two_bytes(0xBF0B);        /*memory of monitor for init*/
	send('E');
	send(init_data);
	send('A');
	send_two_bytes(0xBF30);        /*memory of monitor for offset*/
	send('E');
	send((unsigned char)(register_block_address >>8));
}

int load_6811_bootstrap(void)

{
   unsigned char data;
   int i, flag = success;
   unsigned int address = 0x0000, status_and_data;

   unsigned char program[256]= {0x8E,0x00,0xFF,0xC6,0x30,0xF7,0x10,0x2B,
								0xC6,0xC0,0xF7,0x10,0x2C,0xC6,0x2C,0xF7,
								0x10,0x2D,0x20,0x20,0x86,0x55,0xB7,0x10,
								0x3A,0x86,0xAA,0xB7,0x10,0x3A,0xB6,0x10,

								0x2E,0x84,0x20,0x27,0xEF,0xB6,0x10,0x2F,
								0x39,0xF6,0x10,0x2E,0xC4,0xC0,0x27,0xF9,
								0xB7,0x10,0x2F,0x39,0x8D,0xDE,0x81,0x53,
								0x27,0x45,0x81,0x41,0x27,0x39,0x81,0x44,

								0x27,0x1B,0x81,0x45,0x27,0x24,0x81,0x4D,
								0x27,0x19,0x81,0x43,0x27,0x22,0x81,0x50,
								0x27,0x54,0x81,0x4F,0x27,0x32,0x81,0x52,
								0x26,0xDA,0x7E,0xBF,0x00,0xA6,0x00,0x8D,

								0xC8,0x20,0xD1,0xA6,0x00,0x8D,0xC2,0x08,
								0x20,0xCA,0x8D,0xA8,0xA7,0x00,0x20,0xC4,
								0x8D,0xA2,0xA7,0x00,0x08,0x20,0xBD,0x8D,
								0x9B,0x16,0x8D,0x98,0x8F,0x20,0xB5,0x8F,

								0xF7,0x10,0x2F,0x8D,0xA4,0x8F,0x20,0xAC,
								0x8D,0x8A,0xC6,0x20,0xF7,0x10,0x3B,0xA7,
								0x00,0xC6,0x21,0xF7,0x10,0x3B,0x18,0xCE,
								0x04,0x76,0x8D,0x3F,0x7F,0x10,0x3B,0xA6,

								0x00,0xB7,0x10,0x2F,0x20,0x8E,0x9D,0x14,
								0xC6,0x16,0xF7,0x10,0x3B,0xA7,0x00,0xC6,
								0x17,0xF7,0x10,0x3B,0x18,0xCE,0x0B,0x29,
								0x8D,0x21,0x7F,0x10,0x3B,0xC6,0x02,0xF7,

								0x10,0x3B,0xA7,0x00,0xC6,0x03,0xF7,0x10,
								0x3B,0x18,0xCE,0x0B,0x29,0x8D,0x0C,0x7F,
								0x10,0x3B,0xA6,0x00,0xB7,0x10,0x2F,0x08,
								0x7E,0x00,0x34,0x18,0x09,0x26,0xFC,0x39,

								0x53,0x54,0x41,0x43,0x4B,0x20,0x53,0x54,
								0x41,0x43,0x4B,0x20,0x53,0x54,0x41,0x43,
								0x4B,0x20,0x53,0x54,0x41,0x43,0x4B,0x20,
								0x53,0x54,0x41,0x43,0x4B,0x20,0x53,0x54};

   _bios_serialcom(_COM_INIT,0,_COM_CHR8 | _COM_STOP1 |
					_COM_NOPARITY |_COM_1200);

   i = 0;
   printf("Loading bootstrap monitor, please wait...\n\n");
   _bios_serialcom(_COM_RECEIVE,0,0);  /*clear recieve register*/
   send(0xFF);
   while( i < 255)
   {
	  send(program[i]);
	  status_and_data=_bios_serialcom(_COM_RECEIVE,0,0);
	  switch(status_and_data & 0xff00)
	  {
	  case 0x0000 :
		 data = (unsigned char)status_and_data;
		 break;
	  default :
		 flag = fail;
		 return flag;
	  }

	  if (data != program[i])
	  {
		flag = fail;
		return flag;
	  }
	  i++;
   }

   send(0xFF);   /*this is the 256th character*/

   _bios_serialcom(_COM_RECEIVE,0,0);   /*WILL RETURN ERROR BECAUSE BAUD*/
										/*RATE OF SENT CHARACTER CHANGES*/

   _bios_serialcom(_COM_INIT,0,_COM_CHR8 | _COM_STOP1 |
					_COM_NOPARITY |_COM_9600);
   return flag;
}

int load_6811_monitor(void)

{
   int i,flag=success;
   unsigned char data;
   unsigned char program[192]= {0x86,0x00,0xB7,0x10,0x35,0x86,0x12,0xB7,
								0x10,0x39,0x86,0x01,0xB7,0x10,0x3D,0x86,
								0x52,0x20,0x1A,0x86,0x53,0x20,0x16,0x86,
								0x42,0x20,0x12,0x86,0x4F,0x20,0x0E,0x86,

								0x54,0x20,0x0A,0x86,0x49,0x20,0x06,0x86,
								0x4D,0x20,0x02,0x86,0x58,0x18,0x30,0xCE,
								0x10,0x00,0xC6,0x20,0x3A,0x35,0x8F,0x5F,
								0x8F,0xC6,0x30,0xE7,0x2B,0xC6,0xC0,0xE7,

								0x2C,0xC6,0x2C,0xE7,0x2D,0x8D,0x66,0x8D,
								0x55,0x81,0x53,0x27,0x37,0x81,0x41,0x27,
								0x2A,0x81,0x47,0x27,0x39,0x5F,0x81,0x44,
								0x27,0x0F,0x81,0x45,0x27,0x14,0x5C,0x81,

								0x4D,0x27,0x06,0x81,0x43,0x27,0x0B,0x20,
								0xDE,0x18,0xA6,0x00,0x8D,0x3F,0x18,0x3A,
								0x20,0xD5,0x8D,0x2A,0x18,0xA7,0x00,0x18,
								0x3A,0x20,0xCC,0x8D,0x21,0x16,0x8D,0x1E,

								0x18,0x8F,0x20,0xC3,0x18,0x8F,0xE7,0x2F,
								0x8D,0x23,0x18,0x8F,0x20,0xB9,0x18,0x35,
								0xCC,0x00,0x22,0xE3,0x0E,0xED,0x1E,0x1C,
								0x23,0xFF,0x1C,0x25,0xF0,0x3B,0x86,0x55,

								0xA7,0x3A,0x86,0xAA,0xA7,0x3A,0x1F,0x2E,
								0x20,0xF4,0xA6,0x2F,0x39,0x1F,0x2E,0x60,
								0xFC,0xA7,0x2F,0x39,0xFF,0xFF,0xFF,0xFF,
								0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};




   i = 0;
   printf("Loading main monitor, please wait...\n\n");
   send('A');
   send_two_bytes(0xBF00);
   while( i < 192)
   {
	  send('C');
	  send(program[i]);
	  i++;
   }
   send('A');
   send_two_bytes(0xBF00);
   send('D');
   data = receive();
   if (data !=0x86 )
		flag = fail;
   return flag;

}
int load_6811_vectors(void)
{

   unsigned char data;
   int i, flag = success;
   unsigned int address = 0x0000;

	unsigned char vectors[64] ={0x20,0x2D,0x56,0x45,0x43,0x54,0x4F,0x52,
								0x20,0x54,0x41,0x42,0x4C,0x45,0x2D,0x20,
								0x20,0x20,0x20,0x20,0x20,0x20,0xBF,0x2B,
								0xBF,0x1F,0xBF,0x1F,0xBF,0x1F,0xBF,0x1F,

								0xBF,0x13,0xBF,0x1F,0xBF,0x1F,0xBF,0x1F,
								0xBF,0x1F,0xBF,0x1F,0xBF,0x1F,0xBF,0x1F,
								0xBF,0x1F,0xBF,0x23,0xBF,0x27,0xBF,0x17,
								0xBF,0x1B,0xBF,0x00,0xBF,0x00,0xBF,0x00};

   printf("Loading reset vectors, please wait...\n\n");

   i = 0;
   send('A');
   send_two_bytes(0xBFC0);

   while( i < 64)
   {
	  send('C');
	  send(vectors[i]);
	  i++;
   }

   i = 0;
   send('A');
   send_two_bytes(0xFFC0);

   while( i < 64)
   {
	  send('C');
	  send(vectors[i]);
	  i++;
   }
   send('A');
   send_two_bytes(0xFFD6);
   send('D');
   data = receive();
   if (data != 0xBF)
		flag = fail;
   return flag;
}


void find_register_location()
{
	unsigned char data_byte = 0x00;
	clear_rs232();
	send('A');
	send_two_bytes(0xFFD6);     /*these addresses will contain BF if*/
	send('D');                  /*the software monitor is loaded, ff if*/
	data_byte = receive();      /*eeprom monitor, and D6 if expanded mode*/
								/*but unused, and 00 if single chip w/ no rom*/


	if (data_byte == 0xBF)       /*Ram monitor loaded - expanded or test mode*/
	{
			send('A');
			send_two_bytes(0xBF0B);        /*memory of monitor for init*/
			send('D');
			data_byte = receive();
			register_block_address = 0x1000 * (0x0F & data_byte);
			return;
	}

	send('A');                          /*if the mode is bootstrap, the   */
	send_two_bytes(0x0000 + CONFIG);    /*data at this location is 44     */
	send('D');                          /*if the data is not 41, then this*/
	data_byte = receive();              /*location is the config register */
										/*as set up by the rom monitor    */
	if (data_byte != 0x44)
			register_block_address = 0x0000; /*rom monitor register offset*/
	else
			register_block_address = 0x1000; /*bootstrap register offset*/
	return;
}



void find_6811_mode(void)
{
	unsigned char data_byte;

	send('A');
	send_two_bytes(register_block_address + HPRIO);
	send('D');
	data_byte = (receive() & 0x60);
	switch (data_byte)
	{
		case 0x00:
		   _6811mode = single_chip_mode;
		   break;
		case 0x20:
		   _6811mode = expanded_mode;
		   break;
		case 0x40:
		   _6811mode = bootstrap_mode;
		   break;
		case 0x60:
		   _6811mode = test_mode;
		   break;
	}
}

int setup_6811_monitor(void)
{
	int flag = success, monitor_ram, vector_ram;
	int character;
	unsigned int status_and_data;
	unsigned char data, user_data;
	prom_mode = ram_edit;
	if (_6811mode == no_mode)
	{
		_bios_serialcom(_COM_INIT,0,_COM_CHR8 | _COM_STOP1 |
						_COM_NOPARITY |_COM_9600);

		_clearscreen( _GCLEARSCREEN );
		printf("\n\nChecking for operating monitor...");
		send('D');
		status_and_data=_bios_serialcom(_COM_RECEIVE,0,0);
		if ((status_and_data & 0xff00) == 0x0000)
		{
			find_register_location();
			find_6811_mode();
			return flag;
		}
	}

	_clearscreen( _GCLEARSCREEN );
start:;
	printf("Select 68HC11 operating mode:\n\n");
	printf("1:  Do not change\n\n");

	printf("2:  Single chip mode            Normal debugging in single chip mode\n");
	printf("                                   (Requires monitor in PROM)\n\n");

	printf("3:  Expanded mode               Normal debugging in expanded mode\n");
	printf("                                   (Requires RAM from BF00 - BFFF or\n");
	printf("                                    monitor in PROM at FF00 - FFFF)\n\n");

	printf("4:  Bootstrap mode              Allows programming of EEPROM or EPROM,\n");
	printf("                                but does not allow debugging\n\n");

	printf("5:  Special test mode           Debugging in expanded mode.  Permits\n");
	printf("                                modifying restricted registers and some\n");
	printf("                                other enhanced features.\n");
	printf("6:  Quit                           (Requires RAM from BF00 - BFFF)\n\n");
	character = getch();
	switch (character)
	{
		case '1':       /*  NO CHANGE   */
		if (_6811mode != no_mode)
			return flag;
		else
			goto start;

		case '2':       /*SINGLE CHIP MODE*/
			printf("Reset controller in single chip mode now.\n\n");
			printf("Press any key when ready.\n\n");
			getch();
			_6811mode = single_chip_mode;
			break;
		case '3':       /*EXPANDED MODE*/
			switch(_6811mode)
			{
			case no_mode: case single_chip_mode:
				printf("Reset controller in bootstrap mode now.\n\n");
				printf("Press any key when ready.\n\n");
				getch();
				flag = load_6811_bootstrap();    /*preload monitor into ram */
				if (flag == fail)
				{
					printf("Error loading bootloader.  Please retry.\n\n");
					goto start;
				}
				register_block_address = 0x1000;
			case bootstrap_mode:
				send('A');
				send_two_bytes(0x103C);   /*HPRIO*/
				send('E');                 /*switch to test mode      */
				send(0x65);

				send('A');
				send_two_bytes(register_block_address + CONFIG);
				send('D');
				data = receive();
				if ((data & 0xf0) == 0xf0)  /*if eeprom is at f800-ff00*/
				{
					data = data & 0x7d;     /*move eeprom to 7800-7fff*/
					send('E');              /*turn off rom*/
					send(data);
				}
				else
				if ((data & 0x02) == 0x02)  /*if rom is on*/
				{
					data = data & 0xfd;     /*turn rom off*/
					send('E');
					send(data);
				}
				flag = load_6811_vectors();
				if (flag != success)
				{
					printf("No Ram at FFC0 - FFFF, Reset in expanded mode\n\n");
					printf("Press any key when ready\n\n");
					getch();
					receive();
					register_block_address = 0x0000;
					find_6811_mode();
					if (_6811mode != expanded_mode)
					{
						printf("No monitor loaded in prom.  Load monitor into (e)eprom\n");
						printf("using bootstrap mode\n\n");
						goto start;
					}
					flag = success;
					break;
				}

				load_6811_monitor();
				choose_register_location();
				send('R');                 /*Begin monitor*/
				receive();
			case test_mode:
				send('A');
				send_two_bytes(register_block_address + 0x3C);    /*HPRIO*/
				send('E');                 /*switch to expanded mode */
				send(0x25);
				break;
			}
			break;
		case '4':       /*BOOTSTRAP MODE*/
			switch(_6811mode)
			{
			case bootstrap_mode:
				break;
			default:
				printf("Reset controller in bootstrap mode now.\n\n");
				printf("Press any key when ready.\n\n");
				getch();
				flag = load_6811_bootstrap();
				if (flag == fail)
				{
					printf("Error loading bootloader.  Please retry.\n\n");
					goto start;
				}
				send('A');
				send_two_bytes(0x1035);   /*BPROT*/
				send('E');                 /*enable eeprom programming*/
				send(0x00);
				send('A');
				send_two_bytes(0x103C);   /*HPRIO*/
				send('E');
				send(0x45);                /*disable bootstrap rom and IRV*/
				register_block_address = 0x1000;
				break;
			}
			break;
		case '5':       /*SPECIAL TEST MODE*/
			switch(_6811mode)
			{
			case no_mode: case expanded_mode: case single_chip_mode:
				printf("Reset controller in bootstrap mode now.\n\n");
				printf("Press any key when ready.\n\n");
				getch();
				flag =load_6811_bootstrap();
				if (flag == fail)
				{
					printf("Error loading bootloader.  Please retry\n\n");
					goto start;
				}
				register_block_address = 0x1000;
			case bootstrap_mode:
				send('A');
				send_two_bytes(0x103C);   /*HPRIO*/
				send('E');                 /*switch to test mode      */
				send(0x65);

				send('A');
				send_two_bytes(register_block_address + CONFIG);
				send('D');
				data = receive();
				if ((data & 0xf0) == 0xf0)  /*if eeprom is at f800-ff00*/
				{
					data = data & 0x7d;     /*move eeprom to 7800-7fff*/
					send('E');              /*remove rom*/
					send(data);
				}
				else
				if ((data & 0x02) == 0x02)  /*if rom is on*/
				{
					data = data & 0xfd;     /*turn rom off*/
					send('E');
					send(data);
				}

				flag = load_6811_monitor();
				if (flag != success)
				{
					printf("Requires Ram at BF00 - BFFF\n\n");
					goto start;
				}
				load_6811_vectors();
				choose_register_location();
				send('R');                 /*Begin test mode monitor  */
				receive();
				break;
			}
			break;
		default:
			flag = fail;
			return flag;
   }
   find_6811_mode();
   return flag;
}




/* ********************************************** */
/*      Basic screen display routines             */
/* ********************************************** */

void redraw_screen(void)
{
	clear_rs232();
	_clearscreen( _GCLEARSCREEN );
	_settextposition( 1 , 13 );
	printf("MEMORY CONTENTS\n");
	printf("       0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F\n");
	display_memory();
	display_registers();
	display_breakpoints();
	if (help_mode == on)
	   display_command_summary();
	find_6811_mode();
	_settextposition(21,1);
	printf("MODE>     ");
	switch (_6811mode)
	{
		case single_chip_mode:
			printf("Single chip mode");
			break;
		case expanded_mode:
			printf("Expanded mode");
			break;
		case bootstrap_mode:
			printf("Bootstrap - ");
			switch (prom_mode)
			{
				case ram_edit:
					printf("Ram edit");
					break;
				case eprom_edit:
					printf("Eprom edit");
					break;
				case eeprom_edit:
					printf("EEprom edit");
			}
			break;
		case test_mode:
			printf("Test mode");
	}
	printf("\nSTATUS> ");
	printf("\nFILENAME>\nHELP>");
}

void help_toggle(void)
{
	switch (help_mode)
	{
		case on:
			help_mode = off;
			break;
		case off:
			help_mode = on;
	}
	redraw_screen();
}


/* ********************************************** */
/* Display_memory displays 128 bytes of memory    */
/* beginning at address display_address           */
/* This routine has been optomized for speed      */
/* ********************************************** */


void display_memory(void)
{
   unsigned char row,col,data_byte[16];
   unsigned int status_and_data;

   display_address = (display_address & 0xffC0);
   send('A');
   send_two_bytes(display_address);
   send('D');
   status_and_data=_bios_serialcom(_COM_RECEIVE,0,0);
   switch(status_and_data & 0xff00)
   {
	case 0x0000 :
	  break;
	default :
	  goto error;
   }

   _settextposition(3,1);
   row = 0;
   do
   {
	  print_hex_2byte(display_address + (row) * 0x10);
	  printf("  ");
	  col = 0;

	  do
	  {
		 _bios_serialcom(_COM_SEND,0,'M');
		 status_and_data =_bios_serialcom(_COM_RECEIVE,0,0);
		 if (status_and_data > 255)
			goto error;
		 data_byte[col] = (unsigned char)status_and_data;
		 if (data_byte[col] <= 0x0f)
			printf("0");
		 printf("%X ",data_byte[col]);
		 col++;

	  }  while (col < 16);

	  col = 0;
	  printf(" ");
	  do
	  {
		 if((data_byte[col] > 0x1f) && (data_byte[col] < 0x7f))
			printf("%c",data_byte[col]);
		 else
			printf(".");
		 col++;
	  }  while (col < 16);
	  row++;
	  printf("\n");
   }  while (row < 8);
   display_program();
   if (help_mode == off)
   {
	_settextposition(12,54);
	printf("CONTROLLER REGISTERS");

	send('A');
	send_two_bytes(register_block_address);
	send('D');
	receive();
	row = 0;
	do
	{
	  col = 0;
	  _settextposition(row + 13,col + 49);
	  print_hex_2byte(register_block_address + (row) * 0x08);
	  printf(" ");
	  col = 0;

	  do
	  {
		 _bios_serialcom(_COM_SEND,0,'M');
		 status_and_data =_bios_serialcom(_COM_RECEIVE,0,0);
		 if (status_and_data > 255)
			goto error;
		 data_byte[0] = (unsigned char)status_and_data;
		 if (data_byte[0] <= 0x0f)
			printf("0");
		 printf("%X ",data_byte[0]);
		 col++;

	  }  while (col < 8);
	row++;
	}  while (row < 8);
   }
   return;
error:;
	clear_status();
	printf("RS232 receive error- ");
	print_hex_byte((unsigned char)(status_and_data >> 8));


}

void display_program(void)
{
	int i;
	unsigned char data;
	unsigned int status_and_data;
	_settextposition(18,0);
	printf("PROGRAM -- ");
	send('A');
	send_two_bytes(PC);
	i = 10;
	do
	{
	   send('M');
	   status_and_data =_bios_serialcom(_COM_RECEIVE,0,0);
	   if (status_and_data > 255)
		  goto error;
	   data = (unsigned char)status_and_data;
	   print_hex_byte(data);
	   printf(" ");
	   i--;
	}  while (i >= 0);
	return;
error:;
	clear_status();
	printf("RS232 receive error- ");
	print_hex_byte((unsigned char)(status_and_data >> 8));

}

void display_stack(void)
{
	int i;
	unsigned char stack_data;
	unsigned int status_and_data;
	_settextposition(19,0);
	printf("STACK   -- ");
	send('A');
	send_two_bytes(SP+1);
	i = 10;
	do
	{
	   send('M');
	   status_and_data =_bios_serialcom(_COM_RECEIVE,0,0);
	   if (status_and_data > 255)
		  goto error;
	   stack_data = (unsigned char)status_and_data;
	   print_hex_byte(stack_data);
	   printf(" ");
	   i--;
	}  while (i >= 0);
	return;
error:;
	clear_status();
	printf("RS232 receive error- ");
	print_hex_byte((unsigned char)(status_and_data >> 8));

}




void display_registers()
{
	int i;
	char stack_data;
	unsigned int address;
	_settextposition( 12, 13);
	printf("REGISTER CONTENTS\n");
	printf("Reg A  = ");
	print_hex_byte(ACCA);
	printf("    Reg B  = ");
	print_hex_byte(ACCB);
	printf("    Reg PC = ");
	print_hex_2byte(PC);
	printf("\nReg IX = ");
	print_hex_2byte(IX);
	printf("  Reg IY = ");
	print_hex_2byte(IY);
	printf("  Reg SP = ");
	print_hex_2byte(SP);

	printf("\nS = ");
	if((CCR & 0x80) == 0x80)
	   printf("1");
	else
	   printf("0");

	printf("  X = ");
	if((CCR & 0x40) == 0x40)
	   printf("1");
	else
	   printf("0");

	printf("  H = ");
	if((CCR & 0x20) == 0x20)
	   printf("1");
	else
	   printf("0");

	printf("  I = ");
	if((CCR & 0x10) == 0x10)
	   printf("1");
	else
	   printf("0");

	printf("\nN = ");
	if((CCR & 0x08) == 0x08)
	   printf("1");
	else
	   printf("0");

	printf("  Z = ");
	if((CCR & 0x04) == 0x04)
	   printf("1");
	else
	   printf("0");

	printf("  V = ");
	if((CCR & 0x02) == 0x02)
	   printf("1");
	else
	   printf("0");

	printf("  C = ");
	if((CCR & 0x01) == 0x01)
	   printf("1");
	else
	   printf("0");
	display_stack();

}


void display_breakpoints(void)
{
  if (help_mode == on)
	  return;
  _settextposition(22,57);
  printf("BREAKPOINTS");
  _settextposition(23,51);
  print_hex_2byte(breakpoint[9]);
  printf(" ");
  print_hex_2byte(breakpoint[8]);
  printf(" ");
  print_hex_2byte(breakpoint[7]);
  printf(" ");
  print_hex_2byte(breakpoint[6]);
  printf(" ");
  print_hex_2byte(breakpoint[5]);
  _settextposition(24,51);
  print_hex_2byte(breakpoint[4]);
  printf(" ");
  print_hex_2byte(breakpoint[3]);
  printf(" ");
  print_hex_2byte(breakpoint[2]);
  printf(" ");
  print_hex_2byte(breakpoint[1]);
  printf(" ");
  print_hex_2byte(breakpoint[0]);
}

void display_command_summary(void)
{

  _settextposition(12,56);
  printf("COMMAND SUMMARY");
  switch(_6811mode)
  {

	case bootstrap_mode:
		_settextposition(13,53);
		printf("E - Edit memory         ");
		_settextposition(14,53);
		printf("D - Display memory      ");
		_settextposition(15,53);
		printf("H - Toggle help off     ");
		_settextposition(16,53);
		printf("L - Load program        ");
		_settextposition(17,53);
		printf("M - Change 6811 mode    ");
		_settextposition(18,53);
		printf("P - Toggle Prom mode    ");
		_settextposition(19,53);
		printf("Q - Quit debugger       ");
		_settextposition(20,53);
		printf("Z - Redraw screen       ");
		break;

	default:
		_settextposition(13,53);
		printf("A - Toggle auto update  ");
		_settextposition(14,53);
		printf("B - Modify breakpoints  ");
		_settextposition(15,53);
		printf("E - Edit memory         ");
		_settextposition(16,53);
		printf("D - Display memory      ");
		_settextposition(17,53);
		printf("G - Go.  Run program    ");
		_settextposition(18,53);
		printf("H - Toggle help off     ");
		_settextposition(19,53);
		printf("L - Load program        ");
		_settextposition(20,53);
		printf("M - Change 6811 mode    ");
		_settextposition(21,53);
		printf("Q - Quit debugger       ");
		_settextposition(22,53);
		printf("R - Modify registers    ");
		_settextposition(23,53);
		printf("S - Single step program ");
		_settextposition(24,53);
		printf("T - Trace multiple steps");
		_settextposition(25,53);
		printf("Z - Redraw screen       ");

   }
}

void clear_status(void)
{
	_settextposition(22,1);
	printf("STATUS>                                      ");
	_settextposition(22,11);
}

void clear_mode_status(void)
{
	_settextposition(21,1);
	printf("MODE>                                        ");
	_settextposition(21,11);
}

void clear_help(void)
{
	_settextposition(24,1);
	printf("HELP>                                        ");
	_settextposition(24,11);
}

void clear_command(void)
{
	_settextposition(25,1);
	printf("6811>                                        ");
	_settextposition(25,11);
}

/* ********************************************** */
/*      Byte and two - byte printing routines     */
/* ********************************************** */


void print_hex_2byte(unsigned hex_value)
{
   if(hex_value <= 0x000f)
	  printf("000");
   else
	  if(hex_value <= 0x00ff)
		 printf("00");
	  else
		 if(hex_value <= 0x0fff)
			printf("0");
   printf("%X",hex_value);
}

void print_hex_byte(unsigned char hex_value)
{
   if (hex_value <= 0x0f)
	  printf("0");
   printf("%X",hex_value);
}

/* ********************************************** */
/*      Byte and two - byte input keyboard        */
/*      input routines                            */
/* ********************************************** */

unsigned input_two_bytes ( unsigned *two_byte )
{
  int i = 0;
  unsigned int key_status;
  char instring[4] = "0000";
  char *instring_ptr;
  int character;
  instring_ptr = &instring[0];
  do
  {
	 character = input_hex_character();
	 key_status = character;
	 switch ( character )
	 {
	 case ascii_return :
		goto end;
	 case ' ':
		if (i == 0)
		{
		   goto end;
		}
		break;
	 case ascii_backspace :
		i--;
		if (i >= 0)
		   printf("\b");
		else
		   goto end;
		break;
	 default:
		key_status = ascii_validhex;
		instring[i] = (char)character;
		printf("%c",character);
		i++;
	 }
   } while( i < 4);
   *two_byte = (unsigned)strtol( instring_ptr, &instring_ptr, 16);
end:
	while (i > 0)
	{
	   printf("\b");
	   i--;
	}
   print_hex_2byte(*two_byte);
   return key_status;
}

unsigned input_one_byte ( unsigned char *one_byte )
{
   int i = 0;
   unsigned int key_status;

   char instring[2] = "00";
   char *instring_ptr;
   int character;
   instring_ptr = &instring[0];
   do
   {
	  character = input_hex_character();
	  key_status = character;
	  switch ( character )
	  {
		 case ascii_return :
			goto end;
		 case ' ':
			if (i == 0)
			{
			   goto end;
			}
			break;
		 case ascii_backspace :
			i--;
			if (i >= 0)
			   printf("\b");
			else
			   goto end;
			break;
		 case ascii_arrow :

			character = getch();
			key_status = character;
			switch (character)
			{
				case ascii_left_arrow :  case ascii_up_arrow :
				case ascii_right_arrow : case ascii_down_arrow  :
				   goto end;
				default:
					key_status = ascii_space;
					goto end;
			}
			break;
		 default:
			key_status = ascii_validhex;
			instring[i] = (char)character;
			printf("%c",character);
			i++;
	  }
   }  while( i < 2);
   *one_byte = (unsigned char)strtol( instring_ptr, &instring_ptr, 16);
end:
	while (i > 0)
	{
	   printf("\b");
	   i--;
	}
   print_hex_byte(*one_byte);
   return key_status;
}

int input_hex_character(void)
{
   int character;
   do
   {
	  character = getch();
   }  while((isxdigit(character) == 0) && (character != ascii_backspace) &&
			(character != ' ') && (character != ascii_return ) &&
			(character !=ascii_arrow));
   return character;
}

unsigned input_two_digits ( unsigned char *two_digits )
{
   int i = 0;
   unsigned int key_status;

   char instring[2] = "00";
   char *instring_ptr;
   int character;
   instring_ptr = &instring[0];
   do
   {
	  character = input_decimal_character();
	  key_status = character;
	  switch ( character )
	  {
		 case ascii_return :
			goto end;
		 case ' ':
			if (i == 0)
			{
			   goto end;
			}
			break;
		 case ascii_backspace :
			i--;
			if (i >= 0)
			   printf("\b");
			else
			   goto end;
			break;
		 case ascii_arrow :

			character = getch();
			key_status = character;
			switch (character)
			{
				case ascii_left_arrow :  case ascii_up_arrow :
				case ascii_right_arrow : case ascii_down_arrow  :
				   goto end;
				default:
					key_status = ascii_space;
					goto end;
			}
			break;
		 default:
			key_status = ascii_decimal;
			instring[i] = (char)character;
			printf("%c",character);
			i++;
	  }
   }  while( i < 2);
   *two_digits = (unsigned char)strtol( instring_ptr, &instring_ptr, 10);
end:
	while (i > 0)
	{
	   printf("\b");
	   i--;
	}
   return key_status;
}

int input_decimal_character(void)
{
   int character;
   do
   {
	  character = getch();
   }  while((isdigit(character) == 0) && (character != ascii_backspace) &&
			(character != ' ') && (character != ascii_return ) &&
			(character !=ascii_arrow));
   return character;
}




/* ********************************************** */
/* RS 232 com port routines.                      */
/*                                                */
/* ********************************************** */
/*
	receive() returns an integer which indicates the status of the
	RS-232 port, and the data received, if any.  The status bits are
	defined as follows:

	  15    Timed out
	  14    Transmission-shift register empty
	  13    Transmission-hold register empty
	  12    Break detected
	  11    Framing error
	  10    Parity error
	   9    Overrun error
	   8    Data ready

When service is _COM_SEND, bit 15 will be set if data could not be sent.

When service is _COM_RECEIVE, the byte read will be returned in the low-
order bits if the call is successful. If an error occurs, at least one of
the high-order bits will be set.

*/

void send_two_bytes( unsigned two_byte )
{
	/* sends high byte first, then low byte */

	unsigned char hex_value;
	while((receive_status() & 0xf000) != 0x6000)
	{}
	hex_value = (unsigned char)(two_byte & 0x00ff);
	send(hex_value);
	while((receive_status() & 0xf000) != 0x6000)
	{}
	hex_value = (unsigned char)((two_byte >> 8) & 0x00ff);
	send(hex_value);
}

void send(unsigned char data_byte)
{
   _bios_serialcom(_COM_SEND,0,data_byte);
}

unsigned char receive()
{
	struct rccoord old_position;
	int status_and_data;
	status_and_data=_bios_serialcom(_COM_RECEIVE,0,0);
	switch(status_and_data & 0xff00)
	{
	case 0x0000 :
	  return (unsigned char)status_and_data;
	default :
	  old_position = _gettextposition();
	  clear_status();
	  printf("RS232 receive error-");
	  _settextposition(22,32);
	  print_hex_byte((unsigned char)(status_and_data >> 8));
	  _settextposition(old_position.row, old_position.col);
	  return 0x00;
	}
}

unsigned receive_status()
{

	return _bios_serialcom(_COM_STATUS,0,0);
}

void clear_rs232(void)
{
	if ((receive_status() & 0x0100) == 0x0100)
	{
		clear_status();
		printf("Extra data received- ");
		print_hex_byte(receive());
		_settextposition(25,11);
	}
}

/* ********************************************************************* */
/* ********************************************************************* */
/* ********************************************************************* */
/* ********************************************************************* */

/* ********************************************************************* */
/* This is the bootstrap program loaded by the                           */
/* subroutine load_6811_bootstrap                                        */
/* ********************************************************************* */
/*

Freeware assember ASxx.EXE Ver 1.03.
0001                         ***************
0002                         *
0003                         *       68HC11 BOOTLOADER PROGRAM
0004                         *
0005                         *       THIS MONITOR PROGRAM IS INTENDED TO BE LOADED
0006                         *       BY THE BOOTLOAD PROCEEDURE IN BOOTSTRAP MODE.
0007                         *       THE PROGRAM PERMITS EDITING EPROM IN THE E9 VERSION, 
0008                         *       AND EEPROM IN ANY 68HC11 MICROCONTROLLER EXCEPT 
0009                         *       THE E20 VERSION.  NOTE THAT THE USER CAN MOVE FROM 
0010                         *       THE BOOTSTRAP MODE TO ANY OTHER MODE BY SOFTWARE, WITHOUT
0011                         *       A SYSTEM RESET.
0012                         *
0013                         *       THIS PROGRAM IS HARD ENCODED INTO THE PROGRAM 6811.C,
0014                         *       THE PC MONITOR PROGRAM, WHICH WILL AUTOMATICALLY LOAD
0015                         *       AND RUN IT, AS WELL AS SWITCH MODES IF DESIRED.
0016                         *
0017                         *       BY:             PHILIP DEVRIES
0018                         *       VERSION:        1.0 
0019                         *       DATE:           DECEMBER 1, 1994
0020                         *       FILENAME:       6811BOOT.ASM
0021                         *       COMPILED BY:    MOTOROLA FREEWARE AS11 CROSS-ASSEMBLER
0022                         *       
0023                         *       MEMORY USE     
0024                         *       BY PROGRAM:     0000 - 00FF
0025                         *       VECTOR TABLE:   BFD6 - BFFF (not used, and not initialized)
0026                         *       CONTROLLER
0027                         *       REGISTER LOC:   1000 - 103F
0028                         *
0029                         *       FOR USE WIT
0030                         *       PC MONITOR
0031                         *       VERSION:        3.X
0032                         *       FILENAME:       6811.C
0033                         *       COMPILED BY:    QUICK-C VERSION 2 BY MICROSOFT
0034                         *       
0035                         ***************
0036                         
0037                         ***************
0038                         *       Equates   
0039                         ***************
0040                         
0041 1000                    REGBS   EQU     $1000           Page location of registers
0042 100e                    TCNT    EQU     REGBS+$0E       Timer counter
0043 101e                    TOC5    EQU     REGBS+$1E       OC5 Timer
0044 1022                    TMSK1   EQU     REGBS+$22       Timer mask 1
0045 1023                    TFLG1   EQU     REGBS+$23       Timer interrupt flags 1
0046 1024                    TMSK2   EQU     REGBS+$24       Timer mask 2
0047 1025                    TFLG2   EQU     REGBS+$25       Timer interrupt flags 2
0048 102b                    BAUD    EQU     REGBS+$2B       SCI baud reg
0049 102c                    SCCR1   EQU     REGBS+$2C       SCI control1 reg
0050 102d                    SCCR2   EQU     REGBS+$2D       SCI control2 reg
0051 102e                    SCSR    EQU     REGBS+$2E       SCI status reg
0052 102f                    SCDR    EQU     REGBS+$2F       SCI data reg
0053 1035                    BPROT   EQU     REGBS+$35       Block protect
0054 1039                    OPTION  EQU     REGBS+$39       Configuration options
0055 103a                    COPRST  EQU     REGBS+$3A       COP reset register
0056 103b                    PPROG   EQU     REGBS+$3B       EEPROM programming register
0057 103c                    HIPRO   EQU     REGBS+$3C       Mode select
0058 103d                    INIT    EQU     $103D           Memory map register
0059 103f                    CONFIG  EQU     $103F           Startup configuration
0060                         
0061 001f                    STACK   EQU     $001F
0062                         
0063                         ***************
0064                         *       This is the reset location for all resets, cold boot, hard reset,
0065                         *       clock monitor, and COP.  Note that interrupts are disabled
0066                         *       (including XIRQ) on reset.
0067                         ***************
0068                         
0069 0000                            ORG     $0000
0070                                 
0071                         
0072 0000 8e 00 ff                   LDS     #$00FF
0073                         
0074 0003 c6 30                      LDAB    #%00110000      Set the baud rate 9600
0075 0005 f7 10 2b                   STAB    BAUD
0076                                 
0077 0008 c6 c0                      LDAB    #%11000000      Set up number of bits serial to 8
0078 000a f7 10 2c                   STAB    SCCR1
0079                                 
0080 000d c6 2c                      LDAB    #%00101100      Enable SCI interupts,
0081 000f f7 10 2d                   STAB    SCCR2           enable transmit and receive
0082                                 
0083 0012 20 20                      BRA     MAIN
0084                         
0085                         **************
0086                         *       This subroutine waits for an input on the SCI (RS-232)
0087                         *       and returns the data in register A.  The COP timer is
0088                         *       also reset here.
0089                         **************
0090                         
0091 0014 86 55              INCHAR  LDAA    #%01010101
0092 0016 b7 10 3a                   STAA    COPRST 
0093 0019 86 aa                      LDAA    #%10101010
0094 001b b7 10 3a                   STAA    COPRST
0095 001e b6 10 2e                   LDAA    SCSR            Status of SCI
0096 0021 84 20                      ANDA    #%00100000      Pick out bit 5
0097 0023 27 ef                      BEQ     INCHAR          If not set, there was no input
0098 0025 b6 10 2f                   LDAA    SCDR            If is set, load data and return
0099 0028 39                         RTS
0100                         
0101                         **************
0102                         *       This subroutine waits for the transmitter buffer to
0103                         *       empty, then transmits the character from A
0104                         **************
0105                         
0106 0029 f6 10 2e           OUTCHAR LDAB    SCSR    
0107 002c c4 c0                      ANDB    #%11000000
0108 002e 27 f9                      BEQ     OUTCHAR
0109 0030 b7 10 2f                   STAA    SCDR
0110 0033 39                         RTS
0111                         
0112                         **************
0113                         *       This is the main loop of the monitor program
0114                         **************
0115                         
0116 0034 8d de              MAIN    BSR     INCHAR          Get instruction code from pc monitor
0117                         
0118 0036 81 53                      CMPA    #'S             S means send X to pc monitor
0119 0038 27 45                      BEQ     SENDADD
0120                                 
0121 003a 81 41                      CMPA    #'A             A means get X from pc monitor
0122 003c 27 39                      BEQ     GETADDR
0123                                 
0124 003e 81 44                      CMPA    #'D             D means send data to pc monitor
0125 0040 27 1b                      BEQ     DUMPMEM         without increment
0126                                 
0127 0042 81 45                      CMPA    #'E             E means edit memory contents
0128 0044 27 24                      BEQ     EDITMEM         without increment
0129                                 
0130 0046 81 4d                      CMPA    #'M             M means send data to pc monitor
0131 0048 27 19                      BEQ     DUMPIMEM        with increment
0132                         
0133 004a 81 43                      CMPA    #'C             C means change memory contents
0134 004c 27 22                      BEQ     EDITIMEM        with increment
0135                                 
0136 004e 81 50                      CMPA    #'P             P means program eeprom
0137 0050 27 54                      BEQ     EEPROM
0138                         
0139 0052 81 4f                      CMPA    #'O             O means program eprom
0140 0054 27 32                      BEQ     EPROM
0141                                 
0142 0056 81 52                      CMPA    #'R             R means run monitor at BF00
0143 0058 26 da                      BNE     MAIN
0144                                 
0145 005a 7e bf 00                   JMP     $BF00
0146                         
0147                         **************
0148                         *       Send data from memory location X to pc monitor.
0149                         **************
0150                         
0151 005d a6 00              DUMPMEM LDAA    ,X              Read data
0152 005f 8d c8                      BSR     OUTCHAR         Send to pc monitor
0153 0061 20 d1                      BRA     MAIN            
0154                         
0155                         **************
0156                         *       Send data from memory location X to pc monitor.
0157                         *       Increment address pointer
0158                         **************
0159                         
0160                         DUMPIMEM 
0161 0063 a6 00                      LDAA    ,X              Read data
0162 0065 8d c2                      BSR     OUTCHAR         Send to pc monitor
0163 0067 08                         INX
0164 0068 20 ca                      BRA     MAIN            
0165                         
0166                         **************
0167                         *       Receive data from pc monitor, save at memory location X. 
0168                         **************
0169                                                                    
0170 006a 8d a8              EDITMEM BSR     INCHAR          Get data from pc monitor
0171 006c a7 00                      STAA    ,X              Store pc monitor data in memory
0172 006e 20 c4                      BRA     MAIN            command C
0173                         
0174                         **************
0175                         *       Receive data from pc monitor, save at memory location X. 
0176                         *       Increment address pointer
0177                         **************
0178                                                                    
0179                         EDITIMEM 
0180 0070 8d a2                      BSR     INCHAR          Get data from pc monitor
0181 0072 a7 00                      STAA    ,X              Store pc monitor data in memory
0182 0074 08                         INX
0183 0075 20 bd                      BRA     MAIN            command C
0184                         
0185                         **************
0186                         *       Receive address from pc monitor, save in X.
0187                         **************
0188                         
0189 0077 8d 9b              GETADDR BSR     INCHAR          Get 1st character of address (LSBS)
0190 0079 16                         TAB                     Put in B, low address
0191 007a 8d 98                      BSR     INCHAR          Get 2nd character of address (MSBS)
0192                         *                               Put in A, high address        
0193 007c 8f                         XGDX                    Store address from D in X
0194 007d 20 b5                      BRA     MAIN  
0195                         
0196                         **************
0197                         *       Send the current address in X to the pc monitor.
0198                         **************
0199                         
0200 007f 8f                 SENDADD XGDX                    Put X into D
0201 0080 f7 10 2f                   STAB    SCDR            Send LSB's
0202 0083 8d a4                      BSR     OUTCHAR         Send MSB's
0203 0085 8f                         XGDX                    Store address from D in X
0204 0086 20 ac                      BRA     MAIN
0205                         
0206                         **************
0207                         *       EPROM programming for 68HC711E9
0208                         **************
0209                         EPROM
0210 0088 8d 8a                      BSR     INCHAR          This is the data
0211 008a c6 20                      LDAB    #$20
0212 008c f7 10 3b                   STAB    PPROG           Enable eprom latches
0213 008f a7 00                      STAA    ,X              Store data to address
0214 0091 c6 21                      LDAB    #$21            
0215 0093 f7 10 3b                   STAB    PPROG           Enable eeprom programming voltage 
0216 0096 18 ce 04 76                LDY     #1142
0217 009a 8d 3f                      BSR     DELAY           Delay 4mS
0218 009c 7f 10 3b                   CLR     PPROG           Turn off programmer
0219                                 
0220 009f a6 00                      LDAA    ,X
0221 00a1 b7 10 2f                   STAA    SCDR            Send data to monitor
0222                                 
0223 00a4 20 8e                      BRA     MAIN
0224                         
0225                         **************
0226                         *       Routine to program a byte of memory at X
0227                         **************
0228                         
0229                         EEPROM 
0230 00a6 9d 14                      JSR     INCHAR          This is the data
0231                                 
0232 00a8 c6 16                      LDAB    #$16            Control bits to indicate erase byte
0233 00aa f7 10 3b                   STAB    PPROG           Enable programming - disable EEPROM
0234 00ad a7 00                      STAA    ,X              Write random data to address
0235 00af c6 17                      LDAB    #$17            
0236 00b1 f7 10 3b                   STAB    PPROG           Start programmer
0237 00b4 18 ce 0b 29                LDY     #2857           
0238 00b8 8d 21                      BSR     DELAY           Wait 10mS
0239 00ba 7f 10 3b                   CLR     PPROG
0240                                 
0241 00bd c6 02                      LDAB    #$02            Control bits to indicate write byte
0242 00bf f7 10 3b                   STAB    PPROG           Enable programming - disable EEPROM
0243 00c2 a7 00                      STAA    ,X              Send data to EEPROM
0244 00c4 c6 03                      LDAB    #$03
0245 00c6 f7 10 3b                   STAB    PPROG           Start programmer
0246 00c9 18 ce 0b 29                LDY     #2857           
0247 00cd 8d 0c                      BSR     DELAY           Wait 10mS
0248 00cf 7f 10 3b                   CLR     PPROG
0249                                 
0250 00d2 a6 00                      LDAA    ,X              Read data from location X
0251 00d4 b7 10 2f                   STAA    SCDR            Send data back to monitor.  This forces
0252                         *                               the monitor to wait for programmer to
0253                         *                               finish
0254 00d7 08                         INX                     
0255 00d8 7e 00 34                   JMP     MAIN
0256                         
0257                         **************
0258                         *       Delay loop
0259                         **************
0260                         
0261 00db 18 09              DELAY   DEY                     
0262 00dd 26 fc                      BNE     DELAY
0263 00df 39                         RTS
0264                         
0265 00e0                            ORG     $00E0
0266                         
0267 00e0 53 54 41 43 4b 20          FCC     "STACK STACK STACK STACK STACK ST"
     53 54 41 43 4b 20
     53 54 41 43 4b 20
     53 54 41 43 4b 20
     53 54 41 43 4b 20
     53 54


Number of errors 0


*/
/* ********************************************************************* */
/* This is the monitor program loaded in the                             */
/* subroutine load_6811_monitor and also the                             */
/* vectors used in load_6811_vectors                                     */
/* ********************************************************************* */
/*

Freeware assember ASxx.EXE Ver 1.03.
0001                         ***************
0002                         *
0003                         *       PROM VERSION
0004                         *
0005                         *       THIS MONITOR PROGRAM IS INTENDED TO BE LOADED INTO
0006                         *       EEPROM ON THE E2 VERSION OF THE 68HC11.  THIS WILL
0007                         *       PERMIT A MINIMUM DEBUGGING SYSTEM CONSISTING ONLY OF
0008                         *       A 68HC811E2 WITH RS232 LEVEL SHIFTERS.
0009                         *
0010                         *       THIS PROGRAM WILL OPERATE CORRECTLY IF RUN FROM ANY
0011                         *       MEMORY AT FF00 - FFFF, INCLUDING INTERNAL EPROM, ROM, OR
0012                         *       ANY TYPE OF EXTERNAL MEMORY.  IT SHOULD BE NOTED THAT
0013                         *       THE USER WILL WANT THE ABILITY TO CHANGE THE VECTOR
0014                         *       TABLE, AND THEREFORE IT IS NOT RECOMMENDED THAT THIS
0015                         *       PROGRAM BE STORED IN ROM OR EEPROM.
0016                         *
0017                         *       BY:             PHILIP DEVRIES
0018                         *       VERSION:        3.0 
0019                         *       DATE:           DECEMBER 1, 1994
0020                         *       FILENAME:       6811PROM.ASM
0021                         *       COMPILED BY:    MOTOROLA FREEWARE AS11 CROSS-ASSEMBLER
0022                         *       
0023                         *       MEMORY USE     
0024                         *       BY PROGRAM:     FF00 - FFBF
0025                         *       VECTOR TABLE:   FFC0 - FFFF
0026                         *       CONTROLLER
0027                         *       REGISTER LOC:   0000 - 003F (SET BY X OFFSET)
0028                         *
0029                         *       FOR USE WITH
0030                         *       PC MONITOR 
0031                         *       VERSION:        2.X
0032                         *       FILENAME:       6811.C   
0033                         *       COMPILED BY:    QUICK-C VERSION 2 BY MICROSOFT
0034                         *
0035                         ***************
0036                         
0037                         ***************
0038                         *
0039                         *       REGISTER USE:
0040                         *
0041                         *       REGISTER A & B  USED FOR MOVING DATA
0042                         *       REGISTER X      USED AS OFFSET TO STATUS REGISTERS
0043                         *       REGISTER Y      USED AS POINTER TO MEMORY
0044                         *                       AND AS LOCATION TO MOVE STACK POINTER
0045                         *                       TO AND FROM PC MONITOR
0046                         *       TOC5            USE AS 2 BYTE LOCATION FOR MONITOR STACK
0047                         *
0048                         ***************
0049                         
0050                         ***************
0051                         *       Equates   
0052                         ***************
0053                         
0054 000e                    TCNT    EQU     $0E             Timer counter
0055 001c                    TOC4    EQU     $1C             OC4 Timer
0056 001e                    TOC5    EQU     $1E             OC5 Timer
0057 0022                    TMSK1   EQU     $22             Timer mask 1
0058 0023                    TFLG1   EQU     $23             Timer interrupt flags 1
0059 0024                    TMSK2   EQU     $24             Timer mask 2
0060 0025                    TFLG2   EQU     $25             Timer interrupt flags 2
0061 002b                    BAUD    EQU     $2B             SCI baud reg
0062 002c                    SCCR1   EQU     $2C             SCI control1 reg
0063 002d                    SCCR2   EQU     $2D             SCI control2 reg
0064 002e                    SCSR    EQU     $2E             SCI status reg
0065 002f                    SCDR    EQU     $2F             SCI data reg
0066 1035                    BPROT   EQU     $1035           Block protect
0067 1039                    OPTION  EQU     $1039           Configuration options
0068 003a                    COPRST  EQU     $3A             COP reset register
0069 003b                    PPROG   EQU     $3B             EEPROM programming register
0070 103d                    INIT    EQU     $103D           Memory map register
0071 103f                    CONFIG  EQU     $103F           Startup configuration
0072                         
0073 001f                    STACK   EQU     $1F             Timer OC5.  Used for monitor stack
0074                         
0075                         
0076                         **************
0077                         *       Config Register
0078                         *
0079                         *       If chip is 68HC811E2, map EEPROM to F800 - FFFF when reset
0080                         *
0081                         *       Enable the cop, rom, and eeprom, but not security.
0082                         *
0083                         *       Note that this is an eeprom location, that must be programmed
0084                         *       accordingly.
0085                         **************
0086                                 
0087 103f                            ORG     CONFIG                  
0088 103f fb                         FCB     %11111011                   
0089                         
0090                         ***************
0091                         *       This is the reset location for all resets, cold boot, hard reset,
0092                         *       clock monitor, and COP.  
0093                         *
0094                         *       Some of these register changes must happen in the first 64 clock
0095                         *       cycles.
0096                         *
0097                         *       Note that interrupts are disabled (including XIRQ) on reset.  
0098                         *       Internal registers are mapped to page one on reset.
0099                         ***************
0100                         
0101 ff00                            ORG     $FF00
0102                         RESET  
0103                         COP
0104                         CLKMON                                  
0105 ff00 86 00                      LDAA    #%00000000              Enable EEPROM writing
0106 ff02 b7 10 35                   STAA    BPROT                   
0107                                 
0108 ff05 86 12                      LDAA    #%00010010              ADC off, IRQ level sensitive, clock
0109 ff07 b7 10 39                   STAA    OPTION                  monitor disabled, Cop period 262.14mS
0110                                 
0111 ff0a 86 00                      LDAA    #%00000000              Map internal Ram to 0000 and
0112 ff0c b7 10 3d                   STAA    INIT                    registers to 0000
0113                                 
0114 ff0f 86 52                      LDAA    #'R                     Indicates reset
0115 ff11 20 1a                      BRA     INITIAL
0116                         
0117                         **************
0118                         *       These vectors into the monitor program are all technically
0119                         *       interrupt service routines.  
0120                         *       
0121                         *       The initialization character stored in A is used to tell the
0122                         *       pc monitor which type of interrupt occured.
0123                         *
0124                         *       Note that the all maskable interrupts are automatically
0125                         *       disabled here.
0126                         **************
0127                                           
0128 ff13 86 53              TRACE   LDAA    #'S             Indicates singe step break/ timer OC5
0129 ff15 20 16                      BRA     INITIAL  
0130                         
0131 ff17 86 42              SOFTINT LDAA    #'B             Indicates breakpoint interrupt/ SWI
0132 ff19 20 12                      BRA     INITIAL        
0133                         
0134 ff1b 86 4f              ILLOP   LDAA    #'O             Indicates illegal opcode trap
0135 ff1d 20 0e                      BRA     INITIAL  
0136                         
0137 ff1f 86 54              TIMER   LDAA    #'T             Indicates timer interrupt
0138 ff21 20 0a                      BRA     INITIAL  
0139                         
0140 ff23 86 49              MSKINT  LDAA    #'I             Indicates maskable interrupt
0141 ff25 20 06                      BRA     INITIAL  
0142                         
0143 ff27 86 4d              NMI     LDAA    #'M             Indicates nonmaskable interrupt
0144 ff29 20 02                      BRA     INITIAL
0145                         
0146 ff2b 86 58              MONITOR LDAA    #'X             Indicates pc monitor break/ SCI    
0147                         *       BRA     INITIAL
0148                         
0149                         **************
0150                         *       Initialize the offset and set up the monitor stack accordingly.
0151                         *       The user stack is temporarily stored in Y, to be recovered by
0152                         *       the monitor program with the 'S' command.
0153                         **************
0154                         
0155 ff2d 18 30              INITIAL TSY                     Store the user stack in Y
0156                         
0157 ff2f ce 00 00                   LDX     #$0000          This is the register offset
0158                         
0159 ff32 c6 20                      LDAB    #STACK+1
0160 ff34 3a                         ABX                     Put location of OC5 into X
0161 ff35 35                         TXS                     Put into stack pointer
0162 ff36 8f                         XGDX                    Three instructions to 
0163 ff37 5f                         CLRB                    restore offset
0164 ff38 8f                         XGDX
0165                         
0166                         **************
0167                         *       Set up the serial communications port, and send the initial
0168                         *       character in from register A.
0169                         **************
0170                         
0171                         SCI_INTI
0172 ff39 c6 30                      LDAB    #%00110000      Set the baud rate 9600
0173 ff3b e7 2b                      STAB    BAUD,X
0174                                 
0175 ff3d c6 c0                      LDAB    #%11000000      Set up number of bits serial to 8
0176 ff3f e7 2c                      STAB    SCCR1,X
0177                                 
0178 ff41 c6 2c                      LDAB    #%00101100      Enable SCI interupts,
0179 ff43 e7 2d                      STAB    SCCR2,X         enable transmit and receive
0180                         
0181 ff45 8d 66                      BSR     OUTCHAR         Send initialization character
0182                         
0183                         **************
0184                         *       This is the main loop of the monitor program
0185                         **************
0186                         
0187 ff47 8d 55              MAIN    BSR     INCHAR          Get instruction code from pc monitor
0188                         
0189 ff49 81 53                      CMPA    #'S             S means send X to pc monitor
0190 ff4b 27 37                      BEQ     SENDADD
0191                                 
0192 ff4d 81 41                      CMPA    #'A             A means get X from pc monitor
0193 ff4f 27 2a                      BEQ     GETADDR
0194                                 
0195 ff51 81 47                      CMPA    #'G             G means GO!
0196 ff53 27 39                      BEQ     GO
0197                         
0198 ff55 5f                         CLRB                    Number to increment: 0
0199                         
0200 ff56 81 44                      CMPA    #'D             D means send data to pc monitor
0201 ff58 27 0f                      BEQ     DUMPMEM         without increment
0202                                 
0203 ff5a 81 45                      CMPA    #'E             E means edit memory contents
0204 ff5c 27 14                      BEQ     EDITMEM         without increment
0205                                 
0206 ff5e 5c                         INCB                    Number to increment: 1
0207                                 
0208 ff5f 81 4d                      CMPA    #'M             M means send data to pc monitor
0209 ff61 27 06                      BEQ     DUMPMEM         with increment
0210                         
0211 ff63 81 43                      CMPA    #'C             C means change memory contents
0212 ff65 27 0b                      BEQ     EDITMEM         with increment
0213                                 
0214 ff67 20 de                      BRA     MAIN            Invalid instruction code is ignored
0215                         
0216                         **************
0217                         *       Send data from memory location X to pc monitor.
0218                         **************
0219                         
0220 ff69 18 a6 00           DUMPMEM LDAA    ,Y              Read data
0221 ff6c 8d 3f                      BSR     OUTCHAR         Send to pc monitor
0222 ff6e 18 3a                      ABY                     Increment Y if B = 1
0223 ff70 20 d5                      BRA     MAIN            
0224                         
0225                         **************
0226                         *       Receive data from pc monitor, save at memory location X. 
0227                         **************
0228                                                                    
0229 ff72 8d 2a              EDITMEM BSR     INCHAR          Get data from pc monitor
0230 ff74 18 a7 00                   STAA    ,Y              Store pc monitor data in memory
0231 ff77 18 3a                      ABY                     Increment Y if B = 1
0232 ff79 20 cc                      BRA     MAIN            
0233                         
0234                         **************
0235                         *       Receive address from pc monitor, save in X.
0236                         **************
0237                         
0238 ff7b 8d 21              GETADDR BSR     INCHAR          Get 1st character of address (LSBS)
0239 ff7d 16                         TAB                     Put in B, low address
0240 ff7e 8d 1e                      BSR     INCHAR          Get 2nd character of address (MSBS)
0241                         *                               Put in A, high address        
0242 ff80 18 8f                      XGDY                    Store address from D in Y
0243 ff82 20 c3                      BRA     MAIN  
0244                         
0245                         **************
0246                         *       Send the current address in X to the pc monitor.
0247                         **************
0248                         
0249 ff84 18 8f              SENDADD XGDY                    Put Y into D
0250 ff86 e7 2f                      STAB    SCDR,X          Send LSB's
0251 ff88 8d 23                      BSR     OUTCHAR         Send MSB's
0252 ff8a 18 8f                      XGDY                    Store address from D in Y and return
0253 ff8c 20 b9                      BRA     MAIN            Store address from D in Y and return
0254                         
0255                         **************                
0256                         *       This subroutine serves to start the program for both the GO and
0257                         *       the trace command.  If the command is trace, the monitor has to
0258                         *       enable the I bit of the CCR and send data $08 for the TFLG1 and
0259                         *       TMSK1 register, therebye enabling the OC5 interrupt.
0260                         *       If the command is go, the monitor must send data $00 for TFLG1
0261                         *       and TMSK1, therebye not enabling the OC5 interrupt.
0262                         **************
0263                         
0264 ff8e 18 35              GO      TYS                             Load the stack pointer with the user stack
0265 ff90 cc 00 22                   LDD     #0034                   Use timer OC5 to generate an interrupt
0266 ff93 e3 0e                      ADDD    TCNT,X                  just after the first instruction of the
0267 ff95 ed 1e                      STD     TOC5,X                  user program.  
0268 ff97 1c 23 ff                   BSET    TFLG1,X #%11111111      Clear all timer interrupt flags
0269 ff9a 1c 25 f0                   BSET    TFLG2,X #%11110000
0270 ff9d 3b                         RTI
0271                         
0272                         **************
0273                         *       This subroutine waits for an input on the SCI (RS-232)
0274                         *       and returns the data in register A.  The COP timer is
0275                         *       also reset here.
0276                         **************
0277                         
0278 ff9e 86 55              INCHAR  LDAA    #%01010101              Reset COP monitor
0279 ffa0 a7 3a                      STAA    COPRST,X
0280 ffa2 86 aa                      LDAA    #%10101010
0281 ffa4 a7 3a                      STAA    COPRST,X
0282                                 
0283 ffa6 1f 2e 20 f4                BRCLR   SCSR,X  #%00100000      INCHAR
0284 ffaa a6 2f                      LDAA    SCDR,X
0285 ffac 39                         RTS
0286                         
0287                         **************
0288                         *       This subroutine waits for the transmitter buffer to
0289                         *       empty, then transmits the character from A.
0290                         **************
0291                         
0292 ffad 1f 2e 60 fc        OUTCHAR BRCLR   SCSR,X  #%1100000       OUTCHAR
0293 ffb1 a7 2f                      STAA    SCDR,X
0294 ffb3 39                         RTS
0295                         
0296                         **************
0297                         *       Interrupt and reset vectors.
0298                         *       This is the table location for normal modes.
0299                         **************         
0300                         
0301 ffc0                             ORG    $FFC0
0302 ffc0 20 2d 56 45 43 54           FCC    " -VECTOR TABLE  "
     4f 52 20 54 41 42
     4c 45 20 20
0303 ffd0 20 20 20 20 20 20           FCC    "      "
0304                         
0305                         
0306 ffd6                             ORG    $FFD6
0307 ffd6 ff 2b              VSCI     FDB    MONITOR
0308 ffd8 ff 1f              VSPI     FDB    TIMER
0309 ffda ff 1f              VPAIE    FDB    TIMER
0310 ffdc ff 1f              VPAO     FDB    TIMER
0311 ffde ff 1f              VTOF     FDB    TIMER
0312 ffe0 ff 13              VTOC5    FDB    TRACE
0313 ffe2 ff 1f              VTOC4    FDB    TIMER
0314 ffe4 ff 1f              VTOC3    FDB    TIMER
0315 ffe6 ff 1f              VTOC2    FDB    TIMER
0316 ffe8 ff 1f              VTOC1    FDB    TIMER
0317 ffea ff 1f              VTIC3    FDB    TIMER
0318 ffec ff 1f              VTIC2    FDB    TIMER
0319 ffee ff 1f              VTIC1    FDB    TIMER
0320 fff0 ff 1f              VRTI     FDB    TIMER
0321 fff2 ff 23              VIRQ     FDB    MSKINT 
0322 fff4 ff 27              VXIRQ    FDB    NMI    
0323 fff6 ff 17              VSWI     FDB    SOFTINT
0324 fff8 ff 1b              VILLOP   FDB    ILLOP  
0325 fffa ff 00              VCOP     FDB    COP
0326 fffc ff 00              VCLM     FDB    CLKMON
0327 fffe ff 00              VRST     FDB    RESET 


Number of errors 0


*/
/* ********************************************************************* */
/* ********************************************************************* */
/* ********************************************************************* */
/* ********************************************************************* */

